local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")


function SliceCommander:getOrder()
	_G['SC'].order = _G['SC'].order+1

	return _G['SC'].order
end

function SliceCommander:ArrayPos()
	local returnArray = {}
	local selected = {}
	local cd
	for k, v in pairs(SC.SET.Pos) do
		if string.sub(string.upper(k), -2) == 'CD' then
			cd = ' CD'
		else
			cd = ''
		end
		if not selected[v] then
			selected[v] = _G['SC'].BARS[string.upper(k)]['name'] .. cd
		else
			selected[v] = selected[v] .. ', ' .. _G['SC'].BARS[string.upper(k)]['name'] .. cd
		end
	end
	for key = -20, 20, 1 do
		if key ~= 0 then
			if not selected[key] then
				returnArray[key] = key .. '. ' .. L['**Empty**']
			else
				returnArray[key] = key .. '. ' .. selected[key]
			end
		end
	end
	return returnArray
end

function SliceCommander:getColorCDDescription(spell)
	if string.sub(spell, -2) == 'CD' then
		return string.format(L["The color of %s CD bar"], SC_SPELL[spell])
	else
		return string.format(L["The color of %s bar"], SC_SPELL[spell])
	end
end

function SliceCommander:getColorSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = L["Color"],
		desc = SliceCommander:getColorCDDescription(spell),
		type = "color",
		set = function(info, r, g, b)
			SC.BARS[spell]['obj']:SetStatusBarColor(r, g, b)
			SC.BARS[spell]['color'] = {r, g, b}
		end,
		get = function(info) return unpack(SC.BARS[spell]['color']) end
	}
end

function SliceCommander:getCPColorSetting(type, id)
	return {
		order = SliceCommander:getOrder(),
		name = string.format(L["CP %s"], id),
		desc = L["CP color"],
		type = "color",
		set = function(info, r, g, b)
			SC.SET.CP[type][id] = {r, g, b }
			local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)
			SC.BARS['CP']['obj'].combos[id].bg:SetVertexColor(unpack(SC.SET.CP[maxCP][id]))
		end,
		get = function(info) return unpack(SC.SET.CP[type][id]) end
	}
end

function SliceCommander:getPosSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = SliceCommander:getShowCDNamePosition(spell),
		desc = string.format(L["The place where to display the %s bar"], SC_SPELL[spell]),
		type = "select",
		width = "full",
		values = SliceCommander:ArrayPos(),
		set = function(info,val)
			SC.SET.Pos[spell] = val
		end,
		get = function(info) return SC.SET.Pos[spell] end
	}
end

function SliceCommander:getShowCDNamePosition(spell)
	if string.sub(spell, -2) == 'CD' then
		return L["CD Bar position"]
	else
		return L["Bar position"]
	end
end

function SliceCommander:getShowCDName(spell)
	if string.sub(spell, -2) == 'CD' then
		return L["Show CD bar"]
	else
		return L["Show bar"]
	end
end

function SliceCommander:getShowCDDescription(spell)
	if SC_SPELL[spell] == nil then print(spell) end
	if string.sub(spell, -2) == 'CD' then
		return string.format(L["Display the %s cooldown bar"], SC_SPELL[spell])
	else
		return string.format(L["Display the %s timer bar"], SC_SPELL[spell])
	end
end

function SliceCommander:getShowSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = SliceCommander:getShowCDName(spell),
		desc = SliceCommander:getShowCDDescription(spell),
		type = "toggle",
		set = function(info,val)
			SC.SET.display[spell] = val
		end,
		get = function(info) return SC.SET.display[spell] end
	}
end

function SliceCommander:getHeaderSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = string.format(L["%s Settings"], SC_SPELL[spell]),
		type = "header"
	}
end

function SliceCommander:getDisplayDescriptionSetting()
	return {
		order = SliceCommander:getOrder(),
		name = L["displayDescription"],
		fontSize = "medium",
		type = "description"
	}
end

function SliceCommander:getSpellSettings(spell)
	local args = {
		Show = SliceCommander:getShowSetting(spell),
		Color = SliceCommander:getColorSetting(spell),
		Pos = SliceCommander:getPosSetting(spell),
	}
	if SC.SET.SpellSound ~= nil and SC.SET.SpellSound then
		args.SpellSound = SliceCommander:getSpellSound(spell)
	end
	return {
		name = SC_SPELL[spell],
		type = "group",
		order = SliceCommander:getOrder(),
		args = args
	}
end

function SliceCommander:getCDSettings(spell)
	local args = {
		Show = SliceCommander:getShowSetting(spell),
		Color = SliceCommander:getColorSetting(spell),
		Pos = SliceCommander:getPosSetting(spell),
	}
	return {
		name = SC_SPELL[spell],
		type = "group",
		order = SliceCommander:getOrder(),
		args = args
	}
end

function SliceCommander:getRBSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = SC_SPELL[spell],
		desc = string.format(L["%s text."], SC_SPELL[spell]),
		type = "input",
		width = "half",
		set = function(info,val) SC.SET.RB[spell] = val end,
		get = function(info) return SC.SET.RB[spell] end
	}
end
	
function SliceCommander:getSpellCDSettings(spell)
	local args = {
		Show = SliceCommander:getShowSetting(spell),
		Color = SliceCommander:getColorSetting(spell),
		Pos = SliceCommander:getPosSetting(spell),
		CDShow = SliceCommander:getShowSetting(spell..'CD'),
		CDColor = SliceCommander:getColorSetting(spell..'CD'),
		CDPos = SliceCommander:getPosSetting(spell..'CD'),
	}
	if SC.SET.SpellSound ~= nil and SC.SET.SpellSound then
		args.SpellSound = SliceCommander:getSpellSound(spell)
	end
	return {
		name = SC_SPELL[spell],
		type = "group",
		order = SliceCommander:getOrder(),
		args = args
	}
end

function SliceCommander:getSpellSound(spell)
	return {
		name = L["Sound"],
		type = "group",
		order = SliceCommander:getOrder(),
		args = {
			Tick3 = SliceCommander:getSoundSetting(L["Alert - 3 sec"], L["Alert 3 secondes before fade"], spell, 'alert3', "Tick"),
			Tick2 = SliceCommander:getSoundSetting(L["Alert - 2 sec"], L["Alert 2 secondes before fade"], spell, 'alert2', "Tick"),
			Tick1 = SliceCommander:getSoundSetting(L["Alert - 1 sec"], L["Alert 1 secondes before fade"], spell, 'alert1', "Tick"),
			Apply3 = SliceCommander:getSoundSetting(L["Refresh - 3 sec"], L["Refresh 3 secondes before fade"], spell, 'refresh3', "Apply"),
			Apply2 = SliceCommander:getSoundSetting(L["Refresh - 2 sec"], L["Refresh 2 secondes before fade"], spell, 'refresh2', "Apply"),
			Apply1 = SliceCommander:getSoundSetting(L["Refresh - 1 sec"], L["Refresh 1 secondes before fade"], spell, 'refresh1', "Apply"),
			Applied = SliceCommander:getSoundSetting(L["Applied"], string.format(L["%s is applied"], SC_SPELL[spell]), spell, 'applied', "Apply"),
			Expire = SliceCommander:getSoundSetting(L["Expired"], string.format(L["%s buff expire"], SC_SPELL[spell]), spell, 'expired', "Fail")
		}
	}
end

function SliceCommander:getEnergyMarkerSetting(spec, id)
	local maxi = 150
	if spec == 1 then
		maxi = 170
	end
	return {
		order = SliceCommander:getOrder(),
		name = string.format(L["Energy Marker %s"],id),
		type = "range",
		min = 0,
		max = maxi,
		step = 1,
		bigStep = 1,
		set = function(info,val)
			SC.SET.Energy[spec][id] = val
			SliceCommander:SetSparks()
		end,
		get = function(info) return SC.SET.Energy[spec][id] end,
		isPercent = false
	}
end

function SliceCommander:getEnergyMarkerSettings(spec)
	return {
		order = SliceCommander:getOrder(),
		name = L["Energy bar Settings"],
		type = "group",
		args = {
			Energy1 = SliceCommander:getEnergyMarkerSetting(spec, 1),
			Energy2 = SliceCommander:getEnergyMarkerSetting(spec, 2),
			Energy3 = SliceCommander:getEnergyMarkerSetting(spec, 3),
		}
	}
end

function SliceCommander:getSoundSetting(name, description, spell, id, type)
	return {
		order = SliceCommander:getOrder(),
		name = name,
		desc = description,
		type = "select",
		values = function()
			return SliceCommander:ConfigSoundMenuInit(type)
		end,
		set = function(info,val)
			SliceCommander:SoundTest(val)
			SC.BARS[spell]['sound'][id] = val
		end,
		get = function(info)
			if SC.BARS[spell]['sound'] == nil then SC.BARS[spell]['sound'] = {} end
			if SC.BARS[spell]['sound'][id] == nil then SC.BARS[spell]['sound'][id] = 'None' end
			return SC.BARS[spell]['sound'][id]
		end
	}
end

function SliceCommander:InitTable()
	local SC_OptionsTable = {}

	SC_OptionsTable["Sound"] = {
		name = L["Sound settings"],
		handler = SliceCommander,
		type = "group",
		childGroups = "tab",
		args = {
		SoundCombat = {
			order = SliceCommander:getOrder(),
			name = L["Sound Combat"],
			desc = L["Disable sounds out of combat"],
			type = "toggle",
			set = function(info,val) SC.SET.SoundCombat = val end,
			get = function(info) return SC.SET.SoundCombat end
		},
		SoundChannel = {
			order = SliceCommander:getOrder(),
			name = L["Sound channel"],
			desc = L["Choose the sound channel with which you want to play SliceCommander sounds. 'MASTER' must be use if you want to ear sounds when sound is disable. Other channels should be use if you want o lower volume."],
			type = "select",
			values = function()
				local returnArray = {
					["Master"] = "Master",
					["SFX"] = "SFX",
					["Ambience"] = "Ambience",
					["Music"] = "Music",
				}
				return returnArray
			end,
			set = function(info,val)
				SC.SET.soundChannel = val
			end,
			get = function(info) return SC.SET.soundChannel end
		},
		SoundVolume = {
			order = SliceCommander:getOrder(),
			name = L["Sound Volume"],
			desc = L["Warning: It will change the sound channel volume. If other sounds are played with this channel it will also change their volume."],
			type = "range",
			min = 0,
			max = 1,
			step = 0.05,
			bigStep = 0.1,
			set = function(info,val)
				SliceCommander:ChangeVolume(val)
			end,
			get = function(info) return tonumber(SliceCommander:GetVolume()) end,
			isPercent = true
		},
		SoundSpell = {
			order = SliceCommander:getOrder(),
			name = L["Enable spell sound notitication"],
			desc = L["Each spell will have full sound personalization in their respective settings."],
			type = "toggle",
			set = function(info,val)
				SC.SET.SpellSound = val
				SliceCommander:StaticPopup_Show('CONFIG_RL')
			end,
			get = function(info)
				if SC.SET.SpellSound == nil then
					SC.SET.SpellSound = false
				end
				return SC.SET.SpellSound
			end
		},
		}
	}

	SC_OptionsTable["General"] = {
		name = L["General settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			CV = SliceCommander:getSpellCDSettings('CV'),
			TOT = {
				name = SC_SPELL['TOT'],
				type = "group",
				order = SliceCommander:getOrder(),
				args = {
					TOTShow = SliceCommander:getShowSetting('TOT'),
					TOTPos = SliceCommander:getPosSetting('TOT'),
					TOTColor = SliceCommander:getColorSetting('TOT'),
					TOTTarget = {
						order = SliceCommander:getOrder(),
						name = L["Taget name"],
						desc = L["Write a macro which Tricks the target if it is friendly, if not the target of target. Except if a modifier is down the Tricks will target the player name specify here."],
						type = "input",
						set = function(info,val)
							SC.SET.TOTTarget = val
							SliceCommander:CreateTOTMacro()
						end,
						get = function(info) return SC.SET.TOTTarget end
					},
				}
			},
			FE = SliceCommander:getSpellSettings('FE'),
			AC = SliceCommander:getSpellSettings('AC'),
			CD = SliceCommander:getCDSettings('CD'),
			MD = SliceCommander:getCDSettings('MD'),
			DFA = SliceCommander:getCDSettings('DFA'),
			VAN = SliceCommander:getCDSettings('VAN'),
			KS = SliceCommander:getSpellCDSettings('KS'),
			CS = SliceCommander:getSpellSettings('CS'),
			FOTD = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['FOTD'],
				type = "group",
				args = {
					Show = SliceCommander:getShowSetting('FOTD'),
					Pos = SliceCommander:getPosSetting('FOTD'),
					Color = SliceCommander:getColorSetting('FOTD'),
					Icon = {--Shadow of the destroyer icon
						order = SliceCommander:getOrder(),
						name = string.format(L["%s icon"], SC_SPELL['SOTD']),
						desc = string.format(L["%s icon. It's only display when stack reach 30. There is a counter on it."], SC_SPELL['SOTD']),
						type = "toggle",
						set = function(info,val)
							SC.SET.display.SOTD = val
						end,
						get = function(info)
							return SC.SET.display.SOTD
						end
					},
				}
			},
		}
	}

	SC_OptionsTable["Assassination"] = {
		name = L["Assassination settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			KB = SliceCommander:getSpellCDSettings('KB'),
			BA = SliceCommander:getSpellSettings('BA'),
			ST = SliceCommander:getSpellSettings('ST'),
			RUP = SliceCommander:getSpellSettings('RUP'),
			GAR = SliceCommander:getSpellCDSettings('GAR'),
			EX = SliceCommander:getCDSettings('EX'),
			VEN = SliceCommander:getSpellCDSettings('VEN'),
			ENV = SliceCommander:getSpellSettings('ENV'),
			HEM = SliceCommander:getSpellSettings('HEM'),
			EP = SliceCommander:getSpellSettings('EP'),
			SUB = SliceCommander:getSpellSettings('SUB'),
			SS = SliceCommander:getSpellSettings('SS'),
			SS = SliceCommander:getSpellSettings('DP'),
			EnergySetting = SliceCommander:getEnergyMarkerSettings(1),
			DP = {
				name = string.format(L["%s Settings"], L["Poisons"]),
				type = "group",
				order = SliceCommander:getOrder(),
				args = {
					DPIconShow = {
						order = SliceCommander:getOrder(),
						name = L["Show poison icon"],
						desc = string.format(L["Display the %s icon when the timer left on your poison go under specified time. If you are in combat it only display the icon when it fade out."], SC_SPELL['DP']),
						type = "toggle",
						set = function(info,val)
							SC.SET.display.DPICON = val
						end,
						get = function(info) return SC.SET.display.DPICON end
					},
					DPTimeCheck = {
						order = SliceCommander:getOrder(),
						name = L["Minimal time"],
						desc = L["In minutes"],
						type = "range",
						min = 0,
						max = 59,
						step = 1,
						bigStep = 5,
						set = function(info,val)
							if strmatch(val,"%d") then
								SC.SET.display.DPTIME = val
							else
								SC.SET.display.DPTIME = 10
							end
						end,
						get = function(info) return SC.SET.display.DPTIME end
					},
					DPButtonEnable = {
						order = SliceCommander:getOrder(),
						name = L["Enable poison button"],
						desc = L["Enable poison button has no effect if poison icon is not shown. If it is enable there is two restrinction: You can't click through the area where the icon was when the icon is hide in combat and the button does not work when the icon is shown in combat."],
						type = "toggle",
						set = function(info,val)
							SC.SET.display.DPBUT = val
						end,
						get = function(info) return SC.SET.display.DPBUT end
					},
					DPSound = {
						order = SliceCommander:getOrder(),
						name = string.format(L["%s Sound"], SC_SPELL['DP']),
						desc = string.format(L["%s Sound. It has no effect if %s is not display."], SC_SPELL['DP'], SC_SPELL['DP']),
						type = "select",
						values = function()
							local returnArray = { ["None"] = L["None"] }
							for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
								returnArray[SoundName] = SoundName
							end
							return returnArray
						end,
						set = function(info,val)
							SliceCommander:SoundTest(val)
							SC.SET.DPSound = val
						end,
						get = function(info)
							return SC.SET.DPSound
						end
					},
					displayDescriptionChekcedSpec = {
						order = SliceCommander:getOrder(),
						name = L["displayDescriptionChekcedSpec"],
						fontSize = "medium",
						type = "description"
					},
					Lethal = {
						order = SliceCommander:getOrder(),
						name = L["Lethal poison"],
						type = "select",
						values = function()
							return { [SC_SPELL['AP']] = SC_SPELL['AP'], [SC_SPELL['DP']] = SC_SPELL['DP'], [SC_SPELL['WP']] = SC_SPELL['WP'] }
						end,
						set = function(info,val)
							SC.OTHER['POISON']['obj'].button:SetAttribute("spell1", val)
							SC.SET.poison.mh = val
						end,
						get = function(info)
							return SC.SET.poison.mh
						end
					},
					Utility = {
						order = SliceCommander:getOrder(),
						name = L["Utility poison"],
						type = "select",
						values = function()
							return { [SC_SPELL['CRP']] = SC_SPELL['CRP'], [SC_SPELL['LP']] = SC_SPELL['LP'] }
						end,
						set = function(info,val)
							if val ~= SC_SPELL['LP'] or selected then
								SC.OTHER['POISON']['obj'].button:SetAttribute("spell2", val)
							end
							SC.SET.poison.oh = val
						end,
						get = function(info)
							return SC.SET.poison.oh
						end
					},
				}
			},
		}
	}

	SC_OptionsTable["Outlaw"] = {
		name = L["Outlaw settings"],
		handler = SliceCommander,
		type = "group",
		args = {
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			SND = SliceCommander:getSpellSettings('SND'),
			GS = SliceCommander:getSpellSettings('GS'),
			RB = {
				name = SC_SPELL['RB'],
				type = "group",
				order = SliceCommander:getOrder(),
				args = {
					Show = SliceCommander:getShowSetting('RB'),
					Color = SliceCommander:getColorSetting('RB'),
					Pos = SliceCommander:getPosSetting('RB'),
					replaceName = {
						order = SliceCommander:getOrder(),
						name = L["Replace spell name with each following spell text"],
						desc = L["Each active buff actually up will be displayed in the spell name."],
						type = "toggle",
						set = function(info,val)
							SC.SET.display.RBName = val
						end,
						get = function(info) return SC.SET.display.RBName end
					},
					JR = SliceCommander:getRBSetting('JR'),
					GM = SliceCommander:getRBSetting('GM'),
					SIW = SliceCommander:getRBSetting('SIW'),
					TB = SliceCommander:getRBSetting('TB'),
					BT = SliceCommander:getRBSetting('BT'),
					BS = SliceCommander:getRBSetting('BS')
				}
			},
			BF = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['BF'],
				type = "group",
				args = {
					BFShow = {
						order = SliceCommander:getOrder(),
						name = string.format(L["Show %s icon"], SC_SPELL['BF']),
						desc = string.format(L["Display the %s icon when enbale"], SC_SPELL['BF']),
						width = "full",
						type = "toggle",
						set = function(info,val)
							SC.SET.display.BF = val
						end,
						get = function(info) return SC.SET.display.BF end
					}
				}
			},
			AR = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['AR'],
				type = "group",
				args = {
					ARShow = SliceCommander:getShowSetting('AR'),
					ARPos = SliceCommander:getPosSetting('AR'),
					ARColor = SliceCommander:getColorSetting('AR'),
					ARCDShow = SliceCommander:getShowSetting('ARCD'),
					ARCDColor = SliceCommander:getColorSetting('ARCD'),
					ARMarioMod = {
						order = SliceCommander:getOrder(),
						name = L["Enable Mario mod"],
						desc = L["Enable mario star music during Adrenaline rush buff"],
						type = "toggle",
						set = function(info,val)
							if val then
								SliceCommander:SoundTest('Mario')
								SC.BARS['AR']['sound']['applied'] = 'Mario'
							else
								if SC.previousSound then
									StopSound(SC.previousSound)
								end
								SC.BARS['AR']['sound']['applied'] = 'None'
							end
							SC.SET.ARMarioMod = val
						end,
						get = function(info) return SC.SET.ARMarioMod end
					}
				}
			},
			CB = SliceCommander:getCDSettings('CB'),
			BE = SliceCommander:getCDSettings('BE'),
			KSP = SliceCommander:getCDSettings('KSP'),
			GH = SliceCommander:getCDSettings('GH'),
			OP = SliceCommander:getSpellSettings('OP'),
			PA = SliceCommander:getCDSettings('PA'),
			EnergySetting = SliceCommander:getEnergyMarkerSettings(2),
		}
	}

	SC_OptionsTable["Sublety"] = {
		name = L["Subtlety settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			GB = SliceCommander:getSpellCDSettings('GB'),
			SD = SliceCommander:getSpellCDSettings('SD'),
			SB = SliceCommander:getSpellCDSettings('SB'),
			ES = SliceCommander:getSpellSettings('ES'),
			NB = SliceCommander:getSpellSettings('NB'),
			SOD = SliceCommander:getSpellSettings('SOD'),
			SUB = SliceCommander:getSpellSettings('SUB'),
			SS = SliceCommander:getSpellSettings('SS'),
			EnergySetting = SliceCommander:getEnergyMarkerSettings(3),
		}
	}

	SC_OptionsTable["Complementary"] = {
		name = L["Complementary settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			kickHeader = {
				order = SliceCommander:getOrder(),
				name = L["Kick Feature"],
				type = "group",
				args = {
					kickEnable = {
						order = SliceCommander:getOrder(),
						name = L["Enable"],
						desc = L["If enable, a warning message will be display when you succeed to interrupts spellcasting."],
						type = "toggle",
						set = function(info,val) SC.SET.kickEnable = val end,
						get = function(info) return SC.SET.kickEnable end
					},
					kickChannel = {
						order = SliceCommander:getOrder(),
						name = L["Channel"],
						desc = L["Choose which Channel the success message will be send to. Only one message is send. Then Channel priority order is Instance > Raid > Pary > Say."],
						type = "multiselect",
						values = {
							['INSTANCE_CHAT'] = L["Instance"],
							['RAID'] = L["Raid"],
							['PARTY'] = L["Party"],
							['SAY'] = L["Say"]
						},
						set = function(info, key, val) SC.SET.kickChannel[key] = val end,
						get = function(info, key) return SC.SET.kickChannel[key] end
					},
					kickMessage = {
						order = SliceCommander:getOrder(),
						name = L["Warning message"],
						desc = L["The warning message which will be display when you kick is successful."],
						type = "input",
						width = "full",
						set = function(info,val) SC.SET.kickMessage = val end,
						get = function(info) return SC.SET.kickMessage end
					},
					kickSpell = {
						order = SliceCommander:getOrder(),
						name = L["Add spell"],
						desc = L["Add kicked spell at the start of the message."],
						type = "toggle",
						set = function(info,val) SC.SET.kickSpell = val end,
						get = function(info) return SC.SET.kickSpell end
					},
				}
			},
			CP = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['CP'],
				type = "group",
				args = {
					Bar = SliceCommander:getShowSetting('CP'),
					Pos = SliceCommander:getPosSetting('CP'),
					standart = {
						order = SliceCommander:getOrder(),
						name = string.format(L["%s Settings"], L['Standard']),
						type = "header"
					},
					standardCP1 = SliceCommander:getCPColorSetting(5, 1),
					standardCP2 = SliceCommander:getCPColorSetting(5, 2),
					standardCP3 = SliceCommander:getCPColorSetting(5, 3),
					standardCP4 = SliceCommander:getCPColorSetting(5, 4),
					standardCP5 = SliceCommander:getCPColorSetting(5, 5),
					DS = SliceCommander:getHeaderSetting('DS'),
					DSCP1 = SliceCommander:getCPColorSetting(6, 1),
					DSCP2 = SliceCommander:getCPColorSetting(6, 2),
					DSCP3 = SliceCommander:getCPColorSetting(6, 3),
					DSCP4 = SliceCommander:getCPColorSetting(6, 4),
					DSCP5 = SliceCommander:getCPColorSetting(6, 5),
					DSCP6 = SliceCommander:getCPColorSetting(6, 6),
					ANT = SliceCommander:getHeaderSetting('ANT'),
					ANTCP1 = SliceCommander:getCPColorSetting(8, 1),
					ANTCP2 = SliceCommander:getCPColorSetting(8, 2),
					ANTCP3 = SliceCommander:getCPColorSetting(8, 3),
					ANTCP4 = SliceCommander:getCPColorSetting(8, 4),
					ANTCP5 = SliceCommander:getCPColorSetting(8, 5),
					ANTCP6 = SliceCommander:getCPColorSetting(8, 6),
					ANTCP7 = SliceCommander:getCPColorSetting(8, 7),
					ANTCP8 = SliceCommander:getCPColorSetting(8, 8),
				}
			},
			HEALTH = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['HEALTH'],
				type = "group",
				args = {
					Bar = SliceCommander:getShowSetting('HEALTH'),
					Pos = SliceCommander:getPosSetting('HEALTH'),
					healthPercentage = {
						order = SliceCommander:getOrder(),
						name = L["Show HP Value"],
						desc = L["Show HP value rather than health percentage."],
						type = "toggle",
						set = function(info,val)
							SC.SET.HEALTH.value = val
							SliceCommander:HealthBar('HEALTH')
						end,
						get = function(info) return SC.SET.HEALTH.value end
					},
					healthCombat = {
						order = SliceCommander:getOrder(),
						name = L["Hide out of combat"],
						desc = L["Hide health bar when you are not in combat."],
						type = "toggle",
						set = function(info,val)
							SC.SET.HEALTH.HideCombat = val
							SliceCommander:HealthBar('HEALTH')
						end,
						get = function(info) return SC.SET.HEALTH.HideCombat end
					},
					MidHealthUnder = {
						order = SliceCommander:getOrder(),
						name = L["Sound under 50% health"],
						desc = L["The sound play when your health reach 50% health"],
						type = "select",
						values = function()
							local returnArray = { ["None"] = L["None"] }
							for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
								returnArray[SoundName] = SoundName
							end
							return returnArray
						end,
						set = function(info,val)
							SliceCommander:SoundTest(val)
							SC.SET.HEALTH.MidUnder = val
						end,
						get = function(info)
							return SC.SET.HEALTH.MidUnder
						end
					},
					LowHealthUnder = {
						order = SliceCommander:getOrder(),
						name = L["Sound under 35% health"],
						desc = L["The sound play when your health reach 35% health"],
						type = "select",
						values = function()
							local returnArray = { ["None"] = L["None"] }
							for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
								returnArray[SoundName] = SoundName
							end
							return returnArray
						end,
						set = function(info,val)
							SliceCommander:SoundTest(val)
							SC.SET.HEALTH.LowUnder = val
						end,
						get = function(info)
							return SC.SET.HEALTH.LowUnder
						end
					},
				}
			},
			THEALTH = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['THEALTH'],
				type = "group",
				args = {
					Bar = SliceCommander:getShowSetting('THEALTH'),
					Pos = SliceCommander:getPosSetting('THEALTH'),
					healthPercentage = {
						order = SliceCommander:getOrder(),
						name = L["Show HP Value"],
						desc = L["Show HP value rather than health percentage."],
						type = "toggle",
						set = function(info,val)
							SC.SET.THEALTH.value = val
							SliceCommander:HealthBar('THEALTH')
						end,
						get = function(info) return SC.SET.THEALTH.value end
					},
					healthCombat = {
						order = SliceCommander:getOrder(),
						name = L["Hide out of combat"],
						desc = L["Hide health bar when you are not in combat."],
						type = "toggle",
						set = function(info,val)
							SC.SET.THEALTH.HideCombat = val
							SliceCommander:HealthBar('THEALTH')
						end,
						get = function(info) return SC.SET.THEALTH.HideCombat end
					},
				}
			},
			THREAT = {
				order = SliceCommander:getOrder(),
				name = SC_SPELL['THREAT'],
				type = "group",
				args = {
					Bar = SliceCommander:getShowSetting('THREAT'),
					Pos = SliceCommander:getPosSetting('THREAT'),
					ThreatGroupOnly = {--Show Threat bar only in group
						order = SliceCommander:getOrder(),
						name = L["Threat bar only in group"],
						desc = L["Display the Threat bar only if you are in a group or a raid"],
						type = "toggle",
						width = "full",
						set = function(info,val)
							SC.SET.ThreatGroupOnly = val
						end,
						get = function(info) return SC.SET.ThreatGroupOnly end
					},
				}
			},
		}
	}

	SC_OptionsTable["SliceCommander"] = {
		name = "SliceCommander",
		handler = SliceCommander,
		type = "group",
		args = {--SliceCommander
			IsLocked = {--Lock
				order = SliceCommander:getOrder(),
				name = L["Lock"],
				desc = L["Disable Left Click to Move"],
				type = "toggle",
				set = function(info,val)
					SliceCommander:LockCkecked(val)
				end,
				get = function(info) return SC.SET.IsLocked end
			},
			FullTimer = {--Full timer
				order = SliceCommander:getOrder(),
				name = L["Full Timer"],
				desc = L["Make timer full bar duration"],
				type = "toggle",
				set = function(info,val) SC.SET.FullTimer = val end,
				get = function(info) return SC.SET.FullTimer end
			},
			PadLatency = {--Latency
				order = SliceCommander:getOrder(),
				name = L["Enable Latency"],
				desc = L["Pad Alerts with Latency"],
				type = "toggle",
				set = function(info,val) SC.SET.PadLatency = val end,
				get = function(info) return SC.SET.PadLatency end
			},
			SpellText = {--Spell text
				order = SliceCommander:getOrder(),
				name = L["Spell Text"],
				desc = L["Enable spell text on bars"],
				type = "toggle",
				set = function(info,val) SC.SET.SpellText = val end,
				get = function(info) return SC.SET.SpellText end
			},
			GCD = {--Spell text
				order = SliceCommander:getOrder(),
				name = L["GCD Bar"],
				desc = L["Enable GCD bar"],
				type = "toggle",
				set = function(info,val) SC.SET.displayGCD = val end,
				get = function(info) return SC.SET.displayGCD end
			},
			PixelPerfect = {
				order = SliceCommander:getOrder(),
				name = L["Pixel Perfect"],
				desc = L["Should be enabled. If you change this option state you have to reload interface. This option is automatically disabled after a reload to only force uiscale one time."],
				type = "toggle",
				set = function(info,val)
					SC.SET.PixelPerfect = val
					SliceCommander:StaticPopup_Show('CONFIG_RL')
				end,
				get = function(info) return SC.SET.PixelPerfect end
			},
			skinEnable = {
				order = SliceCommander:getOrder(),
				name = L["Enable border"],
				desc = L["Add a border like the one on Tukui."],
				type = "toggle",
				set = function(info,val)
					SC.SET.skin = val
					SliceCommander:ApplySkin(val)
					SliceCommander:ShowHideBorder(val)
				end,
				get = function(info) return SC.SET.skin end
			},
			headerDisplaySetting = {--CDisplay Setting
				order = SliceCommander:getOrder(),
				name = L["Display Setting"],
				type = "header"
			},
			IconSize = {--Icon size
				order = SliceCommander:getOrder(),
				name = L["Icon size"],
				desc = L["Icon size"],
				type = "range",
				min = 10,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SC.SET.IconSize = val
					SliceCommander:SetIconSize(val)
				end,
				get = function(info) return SC.SET.IconSize end,
				isPercent = false
			},
			Width = {--Width
				order = SliceCommander:getOrder(),
				name = L["Width"],
				desc = L["widthDescription"],
				type = "range",
				min = 125,
				max = 300,
				step = 1,
				bigStep = 5,
				set = function(info,val)
					SC.SET.Width = val
					if SC.SET.skin then
						SliceCommander:ApplySkin(false)
					end
					SliceCommander:SetWidth(val)
					if SC.SET.skin then
						SliceCommander:ApplySkin(true)
					end

				end,
				get = function(info) return SC.SET.Width end,
				isPercent = false
			},
			TimerHeight = {--Timer height
				order = SliceCommander:getOrder(),
				name = L["Timer Height"],
				desc = L["Timer Bar Height"],
				type = "range",
				min = 1,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SC.SET.TimerHeight = val
					SliceCommander:SetTimersHeight(val)
					SliceCommander:ApplySkin(SC.SET.skin)
				end,
				get = function(info) return SC.SET.TimerHeight end,
				isPercent = false
			},
			BarMargin = {--Margin
				order = SliceCommander:getOrder(),
				name = L["Bar Margin"],
				desc = L["Bar Margin"],
				type = "range",
				min = -2,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) SC.SET.BarMargin = val end,
				get = function(info) return SC.SET.BarMargin end,
				isPercent = false
			},
			FrameOpacity = {--Opacity
				order = SliceCommander:getOrder(),
				name = L["Frame opacity"],
				desc = L["Frame opacity"],
				type = "range",
				min = 25,
				max = 100,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SliceCommander:FrameOpacity(val)
					SC.SET.FrameOpacity = val
				end,
				get = function(info) return SC.SET.FrameOpacity end,
				isPercent = false
			},
			BarTexture = {--Texture
				order = SliceCommander:getOrder(),
				name = L["Bar Texture"],
				desc = L["Bar Texture for timer and energy"],
				type = "select",
				values = function()
					local returnArray = {}
					for key, value in pairs(SliceCommander_BarTextures) do
						returnArray[key] = key
					end
					return returnArray
				end,
				set = function(info,val)
					SC.SET.BarTexture = val
					SliceCommander:Config_RetextureBars(SliceCommander:BarTexture())
				end,
				get = function(info) return SC.SET.BarTexture end
			},
			headerEnergySetting = {
				order = SliceCommander:getOrder(),
				name = L["Energy bar Settings"],
				type = "header"
			},
			HideEnergy = {
				order = SliceCommander:getOrder(),
				name = L["Hide Energy bar"],
				desc = L["Hide Energy bar"],
				type = "toggle",
				set = function(info,val)
					if val then
						VTimerEnergy:Hide()
					else
						VTimerEnergy:Show()
					end
					SC.SET.HideEnergy = val
				end,
				get = function(info) return SC.SET.HideEnergy end
			},
			EnergyHeight = {
				order = SliceCommander:getOrder(),
				name = L["Energy Height"],
				desc = L["Energy Bar Height"],
				type = "range",
				min = 1,
				max = 40,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SC.SET.EnergyHeight = val
					SliceCommander:SetEnergyHeight(val)
				end,
				get = function(info) return SC.SET.EnergyHeight end,
				isPercent = false
			},
			EnergyTrans = {
				order = SliceCommander:getOrder(),
				name = L["Transparency (Full)"],
				desc = L["The fade out value of the energy bar when it is full."],
				type = "range",
				min = 0,
				max = 100,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SliceCommander:SetTransparency(val)
					SC.SET.EnergyTrans = val
				end,
				get = function(info) return SC.SET.EnergyTrans end,
				isPercent = false
			},
			EnergyColor = {
				order = SliceCommander:getOrder(),
				name = L["Color"],
				type = "color",
				set = function(info, r, g, b)
					VTimerEnergy:SetStatusBarColor(r, g, b)
					SC.SET.energyColor = {r, g, b}
				end,
				get = function(info) return unpack(SC.SET.energyColor) end
			},
			headerPandemicSetting = {
				order = SliceCommander:getOrder(),
				name = L["Pandemic"],
				type = "header"
			},
			PandemicToggle = {
				order = SliceCommander:getOrder(),
				name = L["Enable pandemic tick"],
				desc = L["A small red bar will be displaid on each timer bar impacted by the mechanic to show you when you can refresh the corresponding buff/debuff."],
				type = "toggle",
				set = function(info,val)
					SC.SET.Pandemic = val
				end,
				get = function(info) return SC.SET.Pandemic end
			},
		}
	}

	return SC_OptionsTable
end

function SliceCommander:ResetOptions()
	SC.SET = nil
	SliceCommander:TestValueOptions()
	if SC.BARS['SND']['obj'] ~= 0 then
		SliceCommander:ApplyConf()
		LibStub("AceConfigRegistry-3.0"):NotifyChange("SliceCommander")
	end
	SliceCommander:StaticPopup_Show('CONFIG_RL')
end

function SliceCommander:InitConfigurationTable()
	local config_table = SliceCommander:InitTable()

	AceConfig:RegisterOptionsTable("SliceCommander", config_table.SliceCommander)
	AceConfig:RegisterOptionsTable("SliceCommander_General", config_table.General)
	AceConfig:RegisterOptionsTable("SliceCommander_Assassination", config_table.Assassination)
	AceConfig:RegisterOptionsTable("SliceCommander_Outlaw", config_table.Outlaw)
	AceConfig:RegisterOptionsTable("SliceCommander_Sublety", config_table.Sublety)
	AceConfig:RegisterOptionsTable("SliceCommander_Complementary", config_table.Complementary)
	AceConfig:RegisterOptionsTable("SliceCommander_Sound", config_table.Sound)

	local SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander","SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_General",L["General settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Assassination",L["Assassination settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Outlaw",L["Outlaw settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Sublety",L["Subtlety settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Complementary",L["Complementary settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Sound",L["Sound settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end
end